/* -*- c++ -*- ***************************************************************/
/* Echtzeitsysteme                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/*          M U L T I _ L E V E L _ Q U E U E _ S C H E D U L E R            */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/* A multi level queue scheduler                                             */
/*****************************************************************************/

#ifndef __multi_level_queue_scheduler_h__
#define __multi_level_queue_scheduler_h__

#include "infra/types.h"

#include "interrupt/guard.h"

#include "thread/dispatch.h"
#include "thread/scheduler_base.h"
#include "thread/thread.h"
#include "thread/thread_queue.h"

class Mutex;

/**
 * \class Multi_Level_Queue_Scheduler
 * \brief A Multi-Level-Queue-Scheduler
 **/
class Multi_Level_Queue_Scheduler
  : public Scheduler_Base, 
    public Dispatcher, 
    public Thread_Queue< 8 >
{

protected:
  
  /**
   * \brief Shall we execute a real context switch or shall we just start
   *        the next context?
   *
   * This flag is need so we can set aside maintaining a thread's state 
   * seperatly. Every time a thread is added to the ready queue its context
   * will be initialized. In the case when a thread has been activated
   * multiple times and finished execution or gets killed we may not just
   * switch the contexts of finished/killed thread and the thread that will
   * be executed next, as we might overwrite the initialized context of
   * finished/killed thread. Therefore we set this flag each time a thread
   * has been activated multiple times and finishes execution or gets killed,
   * in this case the next thread is started by loading its context rather
   * switching between different contexts.
   **/
  bool no_switch;

  /**
   * \brief The current idle thread
   **/
  Thread* current_idle_thread;

  /**
   * \brief Set the idle thread
   * \param thread The idle thread to be set
   **/
  void set_idle_thread(Thread* thread);

  /**
   * \brief Enqueue and init a thread
   * \param thread The thread to be enqueued and initialized
   *
   * this is a little helper function to enqueue a thread into the queue of
   * threads ready for execution and initializing its context previously.
   **/
  void enqueue_init(Thread* thread);

  /**
   * \brief Get the next thread to be executed
   * \return The next thread to be executed
   *
   * Determine the thread with the highest thread within the ready queue.
   **/ 
  Thread* schedule();

  friend class Mutex;

public:

  /**
   * \brief Constructor
   **/  
  Multi_Level_Queue_Scheduler() : no_switch(false) {}

  /**
   * \brief Start the scheduler
   *
   * Start the first thread if there is a thread ready fpr execution otherwise
   * wait for the activation of a thread
   **/
  void start();

  /**
   * \brief Start the next thread
   *
   * Query the next thread ready for execution, that is assigned to the cpu.
   * If there is no thread to be executed wait for the activation of a thread.
   **/
  void reschedule();

  /**
   * \brief Get the current thread
   * \return The thread that is currently executed.
   **/
  Thread* current();

  /**
   * \brief Activate a thread
   * \param thread The thread to be activated
   *
   * Add a thread to the queue of threads, that are ready for execution. The
   * thread will be added as last thread to the queue belonging to the threads
   * priority.
   * A Thread can be activated several times, the order of activations will not
   * be preserved. If a thread has been activated several times and finally
   * finishes execution, the thread will just be enqueued again as often as it
   * has been activated.
   **/ 
  void add(Thread* thread);

  /**
   * \brief Release the processor.
   *
   * A thread releasing the processor will be enqueued as last thread in the
   * queue belonging to its priority.
   **/
  void yield();

  /**
   * \brief Finish execution of the currently running thread.
   *
   * A thread that finishes execution will be started from its beginning, when
   * it is dispatched the cpu the next time.
   * A thread, that has been activated multiple times, will be activated again,
   * when finishing execution. This means it will be enqueued as last thread in
   * the queue belonging to its priority. When the thread is assigned the cpu
   * again it starts from its beginning.
   **/
  void exit();

  /**
   * \brief Kill a thread.
   * \param thread The thread to be killed
   *
   * A thread, that has been killed will be executed from its beginning, when
   * it is assigned the cpu the next time.
   * A thread, that has been activated multiple times, will be activated again,
   * when being killed. This means it will be enqueued as last thread in
   * the queue belonging to its priority. When the thread is assigned the cpu
   * again it starts from its beginning.
   **/
  void kill(Thread* thread);

  /**
   * \brief Restore the thread's original priority
   * \param thread The thread that's priority shall be restored
   * \param prio The priority the thread currently has
   * 
   * This method is used to implement e.g. priority inheritance or priority
   * ceiling protocols. When a thread releases the mutex its original priority
   * has to be restored. This is done by this method.
   **/
  void restore_priority(Thread* thread,prio_type prio);


  /**
   * \brief Inherit a higher priority
   * \param thread The thread inheriting a priority
   * \param prio The priority to be inherited
   *
   * This method is used to implement e.g. priority inheritance or priority
   * ceiling protocols. In certain situations a thread will inherit a priotiy
   * from another thread or mutex. This method takes care of this inheritance
   * stuff. As an example: when priority inheritance is used, the current 
   * owner of a thread inherits the priority of the thread trying to get this
   * mutex, but blocks as this mutex is already taken by another thread.
   **/
  void inherit_priority(Thread* thread,prio_type prio);
};

#endif /* __multi_level_queue_scheduler_h__ */
